var g_sessionToken = null;

var g_matchedBetsUk = null;
var g_matchedBetsAus = null;
var g_unmatchedBetsUk = null;
var g_unmatchedBetsAus = null;

var g_currency = null;
var g_userAgent = null;
var g_loggedIn = false;
var g_excludeBelowPrice = 1;
var g_dateFilterPeriod = "days";
var g_dateFilterType = "placed";
var g_excludeOlderThan = 365;
var g_betsToRetrieve = 50;

// Keep track of prices on both exchanges
var g_currentUkPricesDict = [];
var g_currentAusPricesDict = [];

var g_betsInProgress = null;

// Logging
var g_logLevel = 1;

// API endpoint and response namespace
var g_apiGlobalUrl = "https://api.betfair.com/global/v3/BFGlobalService";
var g_apiUkUrl = "https://api.betfair.com/exchange/v3/BFExchangeService";
var g_apiAusUrl = "https://api-au.betfair.com/exchange/v3/BFExchangeService";

var g_apiNamespace = "http://www.betfair.com/publicapi/BFService/";
var g_siteUrl = "http://www.betfair.com/";

// SOAP envelopes
include("Resources/BetfairSoapEnvelopes.js");

// load wrappers for api objects
include("Resources/BetfairObjects.js");

// load date-time parser (default js one sucks)
include("Resources/date-functions.js");

// load prototype.js
include("Resources/prototype-1.4.0.js");

// set user agent, log settings, kick off login procedure if possible or
// prompt user for login details if not
function begin() {
    g_userAgent = "BetWatcher Widget/" + widget.version;
    log("Identifying as " + g_userAgent);

    timerBetsRefresh.interval = parseInt(preferences.prefTimerInterval.value);
    log("Timer interval set to " + timerBetsRefresh.interval);

    g_betsToRetrieve = parseInt(preferences.prefRecordCount.value);
    log("Bet record count is " + preferences.prefRecordCount.value);

    g_apiGlobalUrl = preferences.prefGlobalUrl.value;
    log("Using " + g_apiGlobalUrl);

    g_apiUkUrl = preferences.prefUkUrl.value;
    log("Using " + g_apiUkUrl);

    g_apiAusUrl = preferences.prefAusUrl.value;
    log("Using " + g_apiAusUrl);

    g_siteUrl = preferences.prefSiteUrl.value;
    log("Site is at " + g_siteUrl);
    
    
    ///////////////////////////////////////////////////////////////////////////
    // configure price filter options
    g_excludeBelowPrice = parseFloat(
        preferences.prefExcludeBelow.value).toFixed(2);
    if (isNaN(g_excludeBelowPrice)) {
        g_excludeBelowPrice = preferences.prefExcludeBelow.value = 1;
    }
    log("Excluding bets under " + g_excludeBelowPrice);    
    
    
    ///////////////////////////////////////////////////////////////////////////
    // configure date filter options
    try {
        g_excludeOlderThan = parseInt(preferences.prefExcludeOlder.value);
    } catch (e) {
        log("Could not parse prefExcludeOver: " + 
            preferences.prefExcludeOlder.value);
    }
    
    g_dateFilterPeriod = preferences.prefFilterDayOrHour.value;
    if (!["days", "hours"].member(g_dateFilterPeriod)) {
        log("Unknown date filter period " + g_dateFilterPeriod 
            + ", defaulting to days");
        g_dateFilterPeriod = "days";
    }
    
    g_dateFilterType = preferences.prefFilterMatchedOrPlaced.value;
    if (!["placed", "matched"].member(g_dateFilterType)) {
        log("Unknown date field " + g_dateFilterType 
            + ", defaulting to placed");
        g_dateFilterType = "placed";
    }
    
    log("Excluding bets " + g_dateFilterType + " more than " 
        + g_excludeOlderThan + " " + g_dateFilterPeriod + " ago");    

        
    ///////////////////////////////////////////////////////////////////////////
    // start login
    setLoginState(false);
    if (preferences.prefUsername.value.length > 0 
                            && preferences.prefPassword.value.length > 0) {
        startLogin();
    } else {
        showWidgetPreferences();
    }
}

function clearBets() {
    g_matchedBetsUk = null;
    g_matchedBetsAus = null;
    g_unmatchedBetsUk = null;
    g_unmatchedBetsAus = null;
}

function logout() {
    setLoginState(false);
    preferences.prefPassword.value = "";
    preferences.prefUsername.value = "";
    clearBets();
    g_currentUkPricesDict = [];
    g_currentAusPricesDict = [];
    displayBets();
}

// issue async login request to API
function startLogin() {
    textFooter.data = "Logging in";
    var url = new URL();
    url.setRequestHeader("User-Agent", g_userAgent);
    url.setRequestHeader("SOAPAction", "login");
    url.setRequestHeader("content-type", "text/xml");
    url.location = g_apiGlobalUrl;
    
    var env = loginEnvelope.replace(/XML_NAMESPACE_PLACEHOLDER/, 
                                                            g_apiNamespace);
    env = env.replace(/PASSWORD_PLACEHOLDER/, preferences.prefPassword.value);
    env = env.replace(/PRODUCT_PLACEHOLDER/, preferences.prefProductId.value);
    env = env.replace(/USERNAME_PLACEHOLDER/, preferences.prefUsername.value);
    url.postData = env;
    
    url.fetchAsync(asyncLoginHandler);
}

// handle login response
function asyncLoginHandler(url) {
    if (url.response = "200") {
        var doc = XMLDOM.parse(url.responseData);
        var loginResp = new LoginResp(doc);
        
        if (loginResp.getErrorCode() == "OK") {
            // save session token and currency
            g_sessionToken = loginResp.getHeader().getSessionToken();
            g_currency = loginResp.getCurrency();
            
            // start timer and start downloading unmatched bets
            setLoginState(true);
            startRefresh();
        } else {
            // something went wrong, kill the timer and log the error
            setLoginState(false);
            log("errorCode: " + loginResp.getHeader().getErrorCode());
            log("subresult: " + loginResp.getErrorCode());
            textFooter.data = loginResp.getErrorCode() + ", " 
                + loginResp.getHeader().getErrorCode();
        }
    } else {
        log("Failed to login: " + url.response);
        textFooter.data = "Failed to login: " + url.response;
        setLoginState(false);
    }
}

function startRefresh() {
    // clear the bet list, and kick off calls against each exchange
    clearBets();

    startGetCurrentBets("U", g_apiUkUrl);
    startGetCurrentBets("U", g_apiAusUrl);
    startGetCurrentBets("M", g_apiUkUrl);
    startGetCurrentBets("M", g_apiAusUrl);
}

// issue async getCurrentBets call to API
function startGetCurrentBets(status, endpoint) {
    textFooter.data = "Getting bets";
    var url = new URL();
    url.setRequestHeader("User-Agent", g_userAgent);
    url.setRequestHeader("SOAPAction", "getCurrentBets");
    url.setRequestHeader("content-type", "text/xml");
    url.location = endpoint;
    
    var env = getCurrentBetsEnvelope.replace(/XML_NAMESPACE_PLACEHOLDER/, 
                                                            g_apiNamespace);
    env = env.replace(/SESSION_PLACEHOLDER/, g_sessionToken);
    env = env.replace(/STATUS_PLACEHOLDER/, status);
    env = env.replace(/COUNT_PLACEHOLDER/, g_betsToRetrieve);
    url.postData = env;
    
    if (status == "U") {
        url.fetchAsync(asyncGetUnmatchedBetsHandler);
    } else {
        url.fetchAsync(asyncGetMatchedBetsHandler);
    }
}

// handle response for unmatched bets
function asyncGetUnmatchedBetsHandler(url) {
    if (!g_loggedIn) return;
    
    if (url.response = "200") {
        var resp = new GetCurrentBetsResp(XMLDOM.parse(url.responseData));
        
        if (resp.getHeader().getErrorCode() == "OK") {
            // save session token
            g_sessionToken = resp.getHeader().getSessionToken();
            
            // save unmatched bets and see if we're ready to display
            if (url.location == g_apiUkUrl) {
                g_unmatchedBetsUk = resp.getBets();
                g_unmatchedBetsUk.each(function(bet, idx) {
                    bet.setExchangeId(1);
                });
            } else if (url.location == g_apiAusUrl) {
                g_unmatchedBetsAus = resp.getBets();
                g_unmatchedBetsAus.each(function(bet, idx) {
                    bet.setExchangeId(2);
                });
            }
            tryDisplay();
        } else {
            // something went wrong, kill the timer and log the error
            setLoginState(false);
            log("errorCode: " + resp.getErrorCode());
            log("subresult: " + resp.getHeader().getErrorCode());
            textFooter.data = resp.getErrorCode() + ", " 
                + resp.getHeader().getErrorCode();
        }
    } else {
        log("Failed to get bets: " + url.response);
        textFooter.data = "Failed to get bets: " + url.response;
        setLoginState(false);
    }
}

// handle response for matched bets
function asyncGetMatchedBetsHandler(url) {
    if (!g_loggedIn) return;

    if (url.response = "200") {
        var resp = new GetCurrentBetsResp(XMLDOM.parse(url.responseData));
        
        if (resp.getHeader().getErrorCode() == "OK") {
            // save session token
            g_sessionToken = resp.getHeader().getSessionToken();
            
            // save matched bets and see if we're ready to display
            if (url.location == g_apiUkUrl) {
                g_matchedBetsUk = resp.getBets();
                g_matchedBetsUk.each(function(bet, idx) {
                    bet.setExchangeId(1);
                });
            } else if (url.location == g_apiAusUrl) {
                g_matchedBetsAus = resp.getBets();
                g_matchedBetsAus.each(function(bet, idx) {
                    bet.setExchangeId(2);
                });
            }
            tryDisplay();
        } else {
            // something went wrong, kill the timer and log the error
            setLoginState(false);
            log("errorCode: " + resp.getErrorCode());
            log("subresult: " + resp.getHeader().getErrorCode());
            textFooter.data = resp.getErrorCode() + ", " 
                + resp.getHeader().getErrorCode();
        }
    } else {
        log("Failed to get bets: " + url.response);
        textFooter.data = "Failed to get bets: " + url.response;
        setLoginState(false);
    }
}

// issue async request to cancel a bet
function startCancelBets(betId, exchangeId) {
    textFooter.data = "Cancelling " + betId;
    var url = new URL();
    url.setRequestHeader("User-Agent", g_userAgent);
    url.setRequestHeader("SOAPAction", "cancelBets");
    url.setRequestHeader("content-type", "text/xml");
    
    // must cancel against correct exchange
    if (exchangeId == 1) {
        url.location = g_apiUkUrl;
        log("Cancelling bet " + betId + " against UK exchange");
    } else if (exchangeId == 2) {
        url.location = g_apiAusUrl;
        log("Cancelling bet " + betId + " against Aus exchange");
    } else {
        var msg = "Unexpected exchange id when cancelling bet: " + exchangeId;
        log(msg);
        textFooter.data = msg;
        return;
    }
        
    cancellationsXML = 
        "<m0:CancelBets>" +
        "    <betId>" + betId + " </betId>" +
        "</m0:CancelBets>";
    
    var env = cancelBetsEnvelope.replace(/XML_NAMESPACE_PLACEHOLDER/, 
                                                            g_apiNamespace);
    env = env.replace(/SESSION_PLACEHOLDER/, g_sessionToken);
    env = env.replace(/CANCELLATIONS_PLACEHOLDER/, cancellationsXML);
    url.postData = env;
    
    url.fetchAsync(asyncCancelBetsHandler);
}

// handle response for bet cancellation
function asyncCancelBetsHandler(url) {
    if (url.response = "200") {
        var doc = XMLDOM.parse(url.responseData);
        
        var cancelBetsResp = new CancelBetsResp(doc);
        if (cancelBetsResp.getHeader().getErrorCode() == "OK") {
            g_sessionToken = cancelBetsResp.getHeader().getSessionToken();
        }
        
        // reload bets so the cancelled stuff disappears
        startRefresh();
        
    } else {
        log("Failed to cancel bets: " + url.response);
        textFooter.data = "Failed to cancel bets: " + url.response;
        setLoginState(false);
    }
}

function tryDisplay() {
    if (g_matchedBetsUk && g_unmatchedBetsUk && g_matchedBetsAus 
                                                && g_unmatchedBetsAus) {
        displayBets();
    } else {
        log("Haven't got all bets yet, postponing display");
    }
}

// combine matched and unmatched bets by bet ID, total up the matched and
// unmatched amounts, and then display
function displayBets() {
    textFooter.data = "Rendering";
    
    ///////////////////////////////////////////////////////////////////////////
    // concat matched and unmatched bets
    log("Concatenating bets");
    var betList = [];
    if (g_unmatchedBetsUk)  betList = betList.concat(g_unmatchedBetsUk);
    if (g_matchedBetsUk)    betList = betList.concat(g_matchedBetsUk);
    if (g_unmatchedBetsAus) betList = betList.concat(g_unmatchedBetsAus);
    if (g_matchedBetsAus)   betList = betList.concat(g_matchedBetsAus);
    
    
    ///////////////////////////////////////////////////////////////////////////
    // Convert age threshold to milliseconds, since date subtraction result
    // is in ms
    var ageThreshold = g_dateFilterPeriod == "days"
        ? g_excludeOlderThan * 24 * 60 * 60 * 1000  // days
        : g_excludeOlderThan * 60 * 60 * 1000;      // hours
        
    var dateNow = new Date();
    
    
    ///////////////////////////////////////////////////////////////////////////
    // loop through bets, combining by bet id and performing date filtering
    combinedBets = {};
    betList.each(function(bet, idx) {
    
        // exclude bets at prices below threshold
        var betStatus = bet.getBetStatus();
        var minPrice = g_excludeBelowPrice;
        
        if (betStatus == "M" && bet.getAvgPrice() < minPrice)
            return;
        else if (betStatus == "U" && bet.getPrice() < minPrice) 
            return;
        
        // exclude bets older than threshold. First we need a simple function
        // to convert the string date to a Date object
        var dateFormatter = function(isoDate) {
            log("parsing date " + isoDate.substr(0, 19));
            return Date.parseDate(isoDate.substr(0, 19), "Y-m-d\\TH:i:s");
        }
        
        // only filter matched bets by matched date, otherwise unmatched bets
        // all get scrapped due to having 00/00/0000 as their matched date
        var filterDate = (g_dateFilterType == "matched" && betStatus == "M")
            ? bet.getMatchedDate(dateFormatter)
            : bet.getPlacedDate(dateFormatter);
        
        var betAge = (dateNow - filterDate);
        log("Age of bet (ms): " + betAge);
        if (filterDate != null && betAge > ageThreshold)
            return;
        
        // create a new entry if required
        var betId = bet.getBetId();
        var marketId = bet.getMarketId();
        var exchangeId = bet.getExchangeId();
        
        // WARNING - this might not be safe, since bet IDs are only unique
        // to each exchange, not across exchanges. In other words, you could
        // have bet 12345 on the UK exchange, and 12345 on the Aus exchange.
        // Therefore this hashtable should be keyed on a combination of bet
        // ID and exchange ID to be absolutely safe.
        if (combinedBets[betId] == null) {
            // we can store the actual bet object here. At most we will have
            // two bet objects with the same id (one matched and one unmatched)
            // but they will share all pertinent data, such as bet type, 
            // market name, selection, etc.
            var currentPrices = (exchangeId == 1)
                    ? g_currentUkPricesDict[marketId]
                    : g_currentAusPricesDict[marketId];
            combinedBets[betId] = 
            {
                'bet'           : bet,
                'unmatched'     : 0,
                'matched'       : 0,
                'totalReturn'   : 0,
                'currentPrices' : currentPrices,
            };
        }
        
        // tally amounts depending on bet status
        if (betStatus == "U") {
            combinedBets[betId].unmatched += bet.getRemainingSize();
        } else if (betStatus == "M") {
            var amountMatched = bet.getMatchedSize();
            combinedBets[betId].matched += amountMatched;
            // tally the total return on the bet - used for average odds
            combinedBets[betId].totalReturn += 
                (amountMatched * bet.getAvgPrice());
        }
    });

    
    ///////////////////////////////////////////////////////////////////////////
    // play alert if bets have changed and alerts are switched on
    var alertChangedBets = parseInt(preferences.prefAlertPartialMatch.value);
    if (alertChangedBets) {
        // any bets that did not get date-filtered and have an unmatched 
        // portion should be compared to the equivalent list from the last 
        // refresh - if any values have changed, play a notification sound
        var betsInProgress = $H(combinedBets).values().select(
                                                        function(bet, idx) {
            return bet.unmatched > 0;
        });
        
        var playAlert = false;
        if (g_betsInProgress != null) {
            log("Checking " + betsInProgress.length + " bets for changes");
            for (var i = 0; i < betsInProgress.length; i++) {
                
                var currentBetValue = betsInProgress[i];
                
                // find this bet in the last snapshot
                var oldBetValue = g_betsInProgress.find(function(bet, idx) {
                    return currentBetValue.bet.getBetId() == bet.bet.getBetId(); 
                });
                
                // if we haven't seen the bet before, play the alert
                if (oldBetValue == null) {
                    log("No previous entry for " + 
                        currentBetValue.bet.getBetId());
                    playAlert = true;
                    break;
                }
                
                log(currentBetValue.bet.getBetId() + ": "
                    + oldBetValue.matched + " -> " 
                    + currentBetValue.matched + " matched");
                    
                // if we found the bet, see if the matched value has changed
                if (oldBetValue.matched != currentBetValue.matched) {
                    playAlert = true;
                    break;
                } else {
                    log(currentBetValue.bet.getBetId() + " has not changed");
                }
            }
        }
        
        if (playAlert) play("Resources/sounds/newmatched.mp3");
        g_betsInProgress = betsInProgress; // save for next time
    }

    
    ///////////////////////////////////////////////////////////////////////////
    // loop through the combined bets and construct a list of the bets to be
    // displayed. By default fully-matched bets are excluded but the user can
    // override this in prefs. The user can also exlclude fully-unmatched bets.
    var includeFullyMatched = 
        parseInt(preferences.prefIncludeFullyMatched.value);
    var includeFullyUnmatched =
        parseInt(preferences.prefIncludeFullyUnmatched.value);
    var displayList = $H(combinedBets).values().select(function(bet, idx) {
        var includeThisBet = true;
        if (bet.unmatched == 0) {
            // fully matched - only include this bet if the appropriate pref
            // is enabled
            includeThisBet = includeFullyMatched;
        } else if (bet.matched == 0) {
            // fully unmatched - only include this bet if the appropriate pref
            // is enabled
            includeThisBet = includeFullyUnmatched;
        }
        
       return includeThisBet;
    });
    
    ///////////////////////////////////////////////////////////////////////////
    // create (or destroy) the right number of frames
    initFrames(displayList.length);
    
    
    ///////////////////////////////////////////////////////////////////////////
    // display bets
    displayList.each(function (betItem, idx) {
        var betObj = betItem.bet;
        var s = betObj.getSelectionName() + ", " 
            + betItem.matched.toString() + ", " 
            + betItem.unmatched.toString();
        
        var totalBetVal = betItem.unmatched + betItem.matched;
        var avgMatched = betItem.totalReturn > 0
            ? betItem.totalReturn / betItem.matched
            : 0;
        var currencySymbol = lookupCurrencySymbol(g_currency);
        var fr = betsFrame.subviews[idx];
        fr.subviews[0].data = betObj.getMarketName() + " / " +
            betObj.getSelectionName();
        fr.subviews[0].color = (betObj.getBetType() == "B") 
            ? "#053686" 
            : "#730404";
        fr.subviews[0].tooltip = betObj.getFullMarketName();
        fr.subviews[0].onMouseUp = createMarketLink(betObj.getMarketId());
        fr.subviews[1].data = 
            currencySymbol + betItem.matched.toFixed(2).toString() + " / " + 
            currencySymbol + totalBetVal.toFixed(2).toString() + " matched" +
            (avgMatched > 0 ? " @ " + formatAvgOdds(avgMatched) : "");
        fr.subviews[1].onMouseUp = createLatestPriceGetter(betObj);
        
        // show latest price if we have it
        fr.subviews[6].data = getLatestPrice(betItem.currentPrices, betObj);
            
        // show progress bar depending on matched amount
        if (betItem.matched > 0) {
            var ratioFilled = betItem.matched / totalBetVal;
            var percentFilled = Math.round(ratioFilled * 100);
            fr.subviews[3].width = (fr.subviews[2].width * ratioFilled) - 2;
            fr.subviews[4].data = percentFilled.toString() + "%";
        } else {
            fr.subviews[3].width = 1;
            fr.subviews[4].data = "0%";
        }
        
        // show cancel button depending on unmatched amount
        if (betItem.unmatched > 0) {
            fr.subviews[5].visible = true;
            fr.subviews[5].src = "Resources/images/deleteBet.gif";
            fr.subviews[5].onMouseUp = createCancelBetHandler(betObj.getBetId(), 
                betObj.getExchangeId());
        } else {
            fr.subviews[5].visible = false;
        }
    });
    textFooter.data = "";
}

// create a closure that disables the other cancel buttons (so
// we can only cancel one bet at a time - saves cocking about
// maintaining the integrity of the session token) and then starts
// the async cancel request
function createCancelBetHandler(id, exchangeId) {
    return function() {
        disableDeleteButtons();
        startCancelBets(id, exchangeId);
    };
}

// create a closure that opens the site to the specified market
// TODO: Aussify!
function createMarketLink(id) {
    return function() {
        openURL(g_siteUrl + "?mi=" + id);
    }
}

// create a closure that gets current prices for the market on which the bet
// is placed, and store them in the global dict for reference in displayBets
function createLatestPriceGetter(bet) {
    return function() {
        textFooter.data = "Getting prices...";
        var url = new URL();
        url.setRequestHeader("User-Agent", g_userAgent);
        url.setRequestHeader("SOAPAction", "getMarketPrices");
        url.setRequestHeader("content-type", "text/xml");
        
        var exchangeId = bet.getExchangeId();
        if (exchangeId == 1) {
            url.location = g_apiUkUrl;
        } else if (exchangeId == 2) {
            url.location = g_apiAusUrl;
        } else {
            log("Unknown exchange id " + exchangeId);
            return;
        }

        var marketId = bet.getMarketId();
        var env = getMarketPricesEnvelope.replace(/XML_NAMESPACE_PLACEHOLDER/, 
                                                            g_apiNamespace);
        env = env.replace(/SESSION_PLACEHOLDER/, g_sessionToken);
        env = env.replace(/CURRENCY_PLACEHOLDER/, g_currency);
        env = env.replace(/LANGUAGE_PLACEHOLDER/, "en-GB");
        env = env.replace(/MARKET_ID_PLACEHOLDER/, marketId);
        url.postData = env;
        url.fetch();
        
        if (url.response = "200") {
            var doc = XMLDOM.parse(url.responseData);
            var pricesResp = new GetMarketPricesResp(doc);
            
            if (pricesResp.getHeader().getErrorCode() == "OK") {
                g_sessionToken = pricesResp.getHeader().getSessionToken();
            }
            
            var marketPrices = pricesResp.getMarketPrices();
            if (exchangeId == 1) {
                g_currentUkPricesDict[marketId] = marketPrices;
            } else { 
                // already validated exchange Id above, so if not UK this must
                // be Aus
                g_currentAusPricesDict[marketId] = marketPrices;
            }
            textFooter.data = "";
            displayBets();
        } else {
            log("Failed to get prices: " + url.response);
            textFooter.data = "Failed to get prices: " + url.response;
            setLoginState(false);
        }
    }
}

function getLatestPrice(marketPrices, bet) {
    if (marketPrices == null) {
        return "";
    }
    
    var marketStatus = marketPrices.getMarketStatus();
    if (marketStatus != "ACTIVE") {
        return marketStatus;
    }
    
    var runnerPrices = marketPrices.findRunnerPrices(bet.getSelectionId(), 
                                                     bet.getAsianLineId());
    if (runnerPrices == null) {
        var msg = "Could not find prices for " + bet.getSelectionName();
        log(msg);
        return msg;
    }

    var bestPrices = (bet.getBetType() == "B")
        ? runnerPrices.getBestPricesToBack()
        : runnerPrices.getBestPricesToLay();
    
    if (bestPrices[0] != null) {
        return "Current price: " + bestPrices[0].getPrice();
    } else {
        return "";
    }
}

// find every row frame showing a cancel button, swap it for the
// greyscale version, and nullify the handler so it can't be clicked
function disableDeleteButtons() {
    if (!betsFrame.subviews) return;
    
    betsFrame.subviews.each(function (fr, idx) {
        if (fr.subviews[5] && fr.subviews[5].visible) {
            fr.subviews[5].src = "Resources/images/disabledDeleteBet.png";
            fr.subviews[5].onMouseUp = null;
        }
    });
}

// create (or destroy) frames until we have the correct number for displaying
// the bets
function initFrames(l) {
    if (!betsFrame.subviews) {
        if (l == 0) {
            //no subviews, nothing to display - bail out
            return;
        }
        
        // we have no subviews at all - create the first one to prep the loop
        var y = 15;
        var fr = createRunnerFrame(y);
        betsFrame.addSubview(fr);
        setupRunnerFrame(fr);
    }
    
    if (betsFrame.subviews.length < l) {
        // create more
        var y = 15 + betsFrame.subviews.length 
            * (betsFrame.subviews[0].height + 10);
        while (betsFrame.subviews.length < l) {
            var fr = createRunnerFrame(y);
            betsFrame.addSubview(fr);
            setupRunnerFrame(fr);
            y += fr.height + 10;
            log("Created frame: " + betsFrame.subviews.length + " exist");
        }
    } else if (betsFrame.subviews.length > l) {
        // destroy some
        while (betsFrame.subviews.length > l) {
            var fr = betsFrame.subviews[betsFrame.subviews.length - 1];
            destroyFrame(fr)
            fr.removeFromSuperview();
            delete fr;
            if (betsFrame.subviews) {
                log("Removed frame: " + betsFrame.subviews.length + " remain");
            } else {
                log("Removed frame: 0 remain");
                break;
            }
        }
    } else {
        // we have the right number - do nothing
    }
}

// remove and destroy each child from the frame
function destroyFrame(fr) {
    while (fr.subviews) {
        sv = fr.subviews[0];
        sv.removeFromSuperview();
        delete sv;
    }
}

// create a frame representing one row of data
function createRunnerFrame(y) {
    var fr = new Frame();
    fr.width = 430; fr.height = 35;
    fr.vOffset = y; fr.hOffset = 5;
    return fr;
}

function setupRunnerFrame(fr) {
    // This is roughly what we create
    //
    // +------------------------------------------------------------+
    // |        |                                 |                 |
    // | CANCEL |  RUNNER NAME (t)                |-----------------|
    // |  BET   |                                 | (img, img1, t2) |
    // | (img2) +---------------------------------+-----------------+
    // |        |  MATCHED DATA (t1)  | CURR (t3) |                 |
    // +------------------------------------------------------------+
    
    // layout vars
    var runnerName = {
        width : ((fr.width / 3) * 2) - 16,
        x : 16,
        y : 15
    };
    
    var matchedAmount = {
        width : ((fr.width / 3) * 2) - 18,
        x : runnerName.x + 2,
        y : 30,
    };
    
    var progressBar = {
        width : fr.width / 3,
        x : runnerName.width,
        y : 0,
    };
    
    // create the controls
    var t = new Text();
    t.hOffset = runnerName.x;
    t.vOffset = runnerName.y;
    t.data = "hello ";
    t.size = 16;
    fr.addSubview(t);
    
    var t1 = new Text();
    t1.width = matchedAmount.width;
    t1.hOffset = matchedAmount.x;
    t1.vOffset = matchedAmount.y;
    t1.size = 12;
    t1.data = "matched ";
    fr.addSubview(t1);
    
    var img = new Image();
    img.src = "Resources/images/emptyProgress.png";
    img.height = 22;
    img.width = progressBar.width;
    img.hOffset = progressBar.x;
    img.vOffset = progressBar.y;
    fr.addSubview(img);
    
    var img1 = new Image();
    img1.src = "Resources/images/filledProgress.png";
    img1.height = 18;
    img1.width = 1;
    img1.hOffset = progressBar.x + 1;
    img1.vOffset = progressBar.y + 2;
    fr.addSubview(img1);
    
    var t2 = new Text();
    t2.width = progressBar.width;
    t2.size = 12;
    t2.hOffset = progressBar.x + (progressBar.width / 2) - 15;
    t2.vOffset = t.vOffset; // align with runner name text
    t2.data = "";
    fr.addSubview(t2);
    
    var img2 = new Image();
    img2.src = "Resources/images/deleteBet.gif";
    img2.hOffset = 0;
    img2.vOffset = 2;
    fr.addSubview(img2);
    
    var t3 = new Text();
    t3.hOffset = t1.hOffset + t1.width;
    t3.vOffset = t1.vOffset + 2;
    t3.size = 12;
    t3.data = "";
    fr.addSubview(t3);
}

// format average odds to either 2 or 4 decimal places, and trim excess 0s
function formatAvgOdds(odds) {
    var s = (odds >= 10) 
        ? odds.toFixed(2).toString() 
        : odds.toFixed(4).toString();
    
    s = s.replace(/0+$/, "");
    return (s.match(/\.$/)) ? s + "0" : s;
}

// convert a currency code (e.g. GBP, USD, EUR) into the appropriate symbol
function lookupCurrencySymbol(currency) {
    var res = "";
    switch(currency) {
        case "GBP":
            res = String.fromCharCode(163);
            break;
        case "EUR":
            res = String.fromCharCode(0x20AC);
            break;
        default:
            res = "";
            break;
    }
    
    return res;
}

function setLoginState(loggedIn) {
    g_loggedIn = loggedIn;
    if (loggedIn) {
        timerBetsRefresh.ticking = true;
        textLogout.visible = true;
        imgLogin.visible = false;
    } else {
        timerBetsRefresh.ticking = false;
        textLogout.visible = false;
        imgLogin.visible = true;
    }
}
