// Wrappers for API objects

// Helpers
function getNodeData(node, element) {
    return node.evaluate("string(" + element + ")");
}

//
// Response header
//
function APIResponseHeader(xmlFragment) {
    this.node = xmlFragment.item(0);
}

APIResponseHeader.prototype.getSessionToken = function() {
    return getNodeData(this.node, "sessionToken");
};

APIResponseHeader.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

APIResponseHeader.prototype.toString = function() {
    return "(header: errorCode=" + this.getErrorCode() + ")";
};

//
// Login response
//
function LoginResp(xmlDoc) {
    this.node = xmlDoc.getElementsByName("n:Result").item(0);
    this.header = new APIResponseHeader(xmlDoc.getElementsByName("header"));
}

LoginResp.prototype.getHeader = function() {
    return this.header;
};

LoginResp.prototype.getCurrency = function() {
    return getNodeData(this.node, "currency");
};

LoginResp.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

LoginResp.prototype.toString = function() {
    return "(LoginResp: " + this.header.toString() + ", currency=" 
        + this.getCurrency() + ")";
};

//
// GetCurrentBets response
//
function GetCurrentBetsResp(xmlDoc) {
    this.node = xmlDoc.getElementsByName("n:Result").item(0);
    this.header = new APIResponseHeader(xmlDoc.getElementsByName("header"));
    
    this.bets = [];
    if (this.header.getErrorCode() == "OK" && this.getErrorCode() == "OK") {
        betNodes = this.node.evaluate("bets/n2:Bet");
        for (var i = 0; i < betNodes.length; i++) {
            this.bets.push(new Bet(betNodes.item(i)));
        }
    }
}

GetCurrentBetsResp.prototype.getHeader = function() {
    return this.header;
};

GetCurrentBetsResp.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

GetCurrentBetsResp.prototype.getTotalRecordCount = function() {
    return parseInt(getNodeData(this.node, "totalRecordCount"));
};

GetCurrentBetsResp.prototype.getBets = function() {
    return this.bets;
};

GetCurrentBetsResp.prototype.toString = function() {
    return "(GetCurrentBetsResp: " + this.header.toString() 
        + ", totalRecordCount=" + this.getTotalRecordCount() 
        + ", bets:" + this.bets.length + ")";
};

//
// CancelBets response
//
function CancelBetsResp(xmlDoc) {
    this.node = xmlDoc.getElementsByName("n:Result").item(0);
    this.header = new APIResponseHeader(xmlDoc.getElementsByName("header"));
}

CancelBetsResp.prototype.getHeader = function() {
    return this.header;
};

CancelBetsResp.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

CancelBetsResp.prototype.toString = function() {
    return "(CancelBetsResp: " + this.header.toString() 
        + ", errorCode: " + this.getErrorCode() + ")";
};

//
// Market response
//
function GetMarketResp(xmlDoc) {
    this.node = xmlDoc.getElementsByName("n:Result").item(0);
    this.header = new APIResponseHeader(xmlDoc.getElementsByName("header"));
    this.market = new Market(xmlDoc.getElementsByName("market"));
}

GetMarketResp.prototype.getHeader = function() {
    return this.header;
};

GetMarketResp.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

GetMarketResp.prototype.toString = function() {
    return "(GetMarketResp: " + this.header.toString() + ")";
};

GetMarketResp.prototype.getMarket = function() {
    return this.market;
}

//
// Market prices response
//
function GetMarketPricesResp(xmlDoc) {
    this.node = xmlDoc.getElementsByName("n:Result").item(0);
    this.header = new APIResponseHeader(xmlDoc.getElementsByName("header"));
    this.marketPrices = 
        new MarketPrices(xmlDoc.getElementsByName("marketPrices"));
}

GetMarketPricesResp.prototype.getHeader = function() {
    return this.header;
};

GetMarketPricesResp.prototype.getErrorCode = function() {
    return getNodeData(this.node, "errorCode");
};

GetMarketPricesResp.prototype.toString = function() {
    return "(GetMarketPricesResp: " + this.header.toString() + ")";
};

GetMarketPricesResp.prototype.getMarketPrices = function() {
    return this.marketPrices;
}

//
// Market
//
function Market(xmlFragment) {
    this.node = xmlFragment.item(0);
    this.runners = [];
    var r = this.node.evaluate("runners/n2:Runner");
    for (var i = 0; i < r.length; i++) {
        this.runners.push(new Runner(r.item(i)));
    }
}

Market.prototype.getRunners = function() {
    return this.runners;
}

//
// Runner
//
function Runner(node) {
    this.node = node;
}

Runner.prototype.getName = function() {
    return getNodeData(this.node, "name");
}

//
// MarketPrices
//
function MarketPrices(xmlFragment) {
    this.node = xmlFragment.item(0);
    this.runnerPrices = [];
    var r = this.node.evaluate("runnerPrices/n2:RunnerPrices");
    for (var i = 0; i < r.length; i++) {
        this.runnerPrices.push(new RunnerPrices(r.item(i)));
    }
    log(this.runnerPrices.length);
}

MarketPrices.prototype.toString = function() {
    return "(RunnerPrices: " + this.runnerPrices.length + ")";
}

MarketPrices.prototype.getMarketId = function() {
    return getNodeData(this.node, "marketId");
}

MarketPrices.prototype.getMarketStatus = function() {
    return getNodeData(this.node, "marketStatus");
}

MarketPrices.prototype.getRunnerPrices = function() {
    return this.runnerPrices;
}

MarketPrices.prototype.findRunnerPrices = function(selectionId, asianLineId) {
    for (var i = 0; i < this.runnerPrices.length; i++) {
        var rp = this.runnerPrices[i];
        if (rp.getSelectionId() == selectionId
        && rp.getAsianLineId() == asianLineId) {
            return rp;
        }
    }
    
    return null;
}

//
// RunnerPrices
//
function RunnerPrices(node) {
    this.node = node;
    var prices = null;
    
    this.bestPricesToBack = [];
    prices = this.node.evaluate("bestPricesToBack/n2:Price");
    for (var i = 0; i < prices.length; i++) {
        this.bestPricesToBack.push(new Price(prices.item(i)));
    }
    
    this.bestPricesToLay = [];
    prices = this.node.evaluate("bestPricesToLay/n2:Price");
    for (var i = 0; i < prices.length; i++) {
        this.bestPricesToLay.push(new Price(prices.item(i)));
    }
}

RunnerPrices.prototype.getSelectionId = function() {
    return getNodeData(this.node, "selectionId");
}

RunnerPrices.prototype.getAsianLineId = function() {
    return getNodeData(this.node, "asianLineId");
}

RunnerPrices.prototype.getBestPricesToBack = function() {
    return this.bestPricesToBack;
}

RunnerPrices.prototype.getBestPricesToLay = function() {
    return this.bestPricesToLay;
}

RunnerPrices.prototype.getSumOfMoneyAvailable = function() {
    var allPrices = this.bestPricesToBack.concat(this.bestPricesToLay);
    var total = 0;
    for (var i = 0; i < allPrices.length; i++) {
        total += allPrices[i].getAmountAvailable();
    }
    
    return total;
}

//
// Price
//
function Price(node) {
    this.node = node;
}

Price.prototype.getAmountAvailable = function() {
    return parseFloat(getNodeData(this.node, "amountAvailable"));
}

Price.prototype.getPrice = function() {
    return parseFloat(getNodeData(this.node, "price"));
}

Price.prototype.toString = function() {
    return "(" + this.getAmountAvailable() + " @ " + this.getPrice() + ")";
}

//
// Bet
//
function Bet(node) {
    this.node = node;
    this.exchangeId = 1; // assume uk
}

Bet.prototype.getAsianLineId = function() {
    return parseInt(getNodeData(this.node, "asianLineId"));
}

Bet.prototype.getBetId = function() {
    return parseInt(getNodeData(this.node, "betId"));
}

Bet.prototype.getBetStatus = function() {
    return getNodeData(this.node, "betStatus");
}

Bet.prototype.getBetType = function() {
    return getNodeData(this.node, "betType");
}

Bet.prototype.getMarketId = function() {
    return getNodeData(this.node, "marketId");
}

Bet.prototype.getMarketName = function() {
    return getNodeData(this.node, "marketName");
}

Bet.prototype.getFullMarketName = function() {
    return getNodeData(this.node, "fullMarketName");
}

Bet.prototype.getSelectionName = function() {
    return getNodeData(this.node, "selectionName");
}

Bet.prototype.getSelectionId = function() {
    return parseInt(getNodeData(this.node, "selectionId"));
}

Bet.prototype.getMatchedSize = function() {
    return parseFloat(getNodeData(this.node, "matchedSize"));
}

Bet.prototype.getPrice = function() {
    return parseFloat(getNodeData(this.node, "price"));
}

Bet.prototype.getRemainingSize = function() {
    return parseFloat(getNodeData(this.node, "remainingSize"));
}

Bet.prototype.getRequestedSize = function() {
    return parseFloat(getNodeData(this.node, "requestedSize"));
}

Bet.prototype.getAvgPrice = function() {
    return parseFloat(getNodeData(this.node, "avgPrice"));
}

Bet.prototype.getPlacedDate = function(parser) {
    return parser(getNodeData(this.node, "placedDate"));
}

Bet.prototype.getMatchedDate = function(parser) {
    return parser(getNodeData(this.node, "matchedDate"));
}

Bet.prototype.setExchangeId = function(exchangeId) {
    this.exchangeId = exchangeId;
}

Bet.prototype.getExchangeId = function() {
    return this.exchangeId;
}

Bet.prototype.toString = function() {
    return "(Bet: betId=" + this.getBetId() + ")";
}

